#!/usr/bin/perl -w

# This script prepares the localscan.conf file.  It is intended
# to be run against a "clean" subnet.  See the README.TXT
# file for more details.
#
# Copyright (C) 1999 by Dylan Greene
# Last modified Wednesday, 19 January, 2000

# Was this script called with a command-line variable?
if ( $ARGV[0] ) {

    # If a filename was specified on the command-line, treat it
    # as the nmap scan data

    open(DATA, "<$ARGV[0]") or die "Can't open $ARGV[0]: $!\n";
    @data = <DATA>;
    close(DATA);
    
    # Get the subnet, mailto, and allclr field values from the user.
    get_particulars();

} else {

    # No data filename was specified on the command line,
    # so the script needs to run nmap itself.

    # This pulls up data from localscan.conf, or runs get_particulars() if that
    # data doesn't exist.
    read_conf();

    foreach $subnet ( split(/\s+/, $subnet_list) ) {
	# Now call nmap with the $subnet variable and store it in an
	# array for processing.

#	$argument = join('',"/usr/local/bin/nmap -sT '",$subnet,"'");
	$argument = join('',$nmap,"/nmap -sT '",$subnet,"'");
	chomp $argument;
	push @data, `$argument`;

    }
}

# Either one of the two options should have retured an array called @data,
# which contains the output of an nmap scan.  The next bit of code processes
# that array.

foreach my $line ( @data ) {
    chomp $line;

    if ( $line =~ /^Interesting/ ) {

	# grab the IP address of the host
	@host_info = split(/\s+/, $line);

	if ( $host_info[4] ) {

	    # If there's DNS info, then
	    $ip = $host_info[4];

	} else {

	    # If there's no DNS info, then
	    $ip = $host_info[3];

	}

	$ip =~ s/\(//;
	$ip =~ s/\)//;
	$ip =~ s/://;

    }
    if ( $line =~ /^[0-9]/ ) {

	# grab any ports reported as open
	@port = split(/\s+/, $line);
	$open_port = $port[0];

	if ( $hosts{$ip} ) {

	    # If this host already has an entry in the %hosts hash,
	    # make sure it doesn't get overwritten.
	    $hosts{$ip} = join (' ', $hosts{$ip},$open_port);

	} else {

	    # Otherwise, create an entry for $ip in %hosts.
	    $hosts{$ip} = $open_port;

	}
    }
}

chomp %hosts;

# Let's start making the localscan.conf file.  The previous
# localscan.conf (if it existed) will be overwritten, but that's
# not a problem because we have all the relevant data from it.

open(OUTPUT, ">localscan.conf");

# Write the value of the "subnet_list", "mailto", "allclr", and "path" fields
# to the new, pristine localscan.conf file.

printf OUTPUT "##### General Information #####\n";
printf OUTPUT "subnet %s\n", $subnet_list;
printf OUTPUT "mailto %s\n", $mailto;
printf OUTPUT "allclr %s\n", $allclr;
printf OUTPUT "nmap %s\n", $nmap;
printf OUTPUT "\n\n##### Hosts\/Ports to Ignore #####\n";

# Now, sort the hash and hand the output to the localscan.conf file
# as "ignore" fields.  The ip_sort() function is explained below.

@keys = keys ( %hosts );
@ordered_keys = sort { ip_sort() } @keys;
foreach $key ( @ordered_keys ) {
    chomp $key;
    printf OUTPUT "ignore %s %s\n", $key, $hosts{$key};
}

close(OUTPUT);


# Now, process the localscan.pl file.  This is unfortunately the only way I
# could see in order to embed hard paths into the file.

# Read it in
open(LOCALSCAN,"<localscan.pl");
@localscan = <LOCALSCAN>;
close(LOCALSCAN);

# Reopen it as a writable file
open(LOCALSCAN,">localscan.pl");

# Process it -- spitting back each line unless that line defines the $path variable.
foreach my $line ( @localscan ) {
    
    if ( $line =~ /^\$path/ ) {
	$path = `pwd`;
	chomp $path;
	printf LOCALSCAN "\$path = \"$path\";\n";

    } else {
	# Make the line pretty, then output it back to the script
	chomp $line;
	$line =~ s/\r$//;
	printf LOCALSCAN "%s\n", $line;
    }
}

close(LOCALSCAN);


##### Subroutines follow ######

# Sorting IP addresses doesn't work too well because
# of the dots, so the ip_sort() subroutine parses them out.

sub ip_sort {

    # Declare the local variables--the sort() function handles
    # this a bit strangely, so we can't just use @_
    my $first_element = $a;
    my $second_element = $b;

    # Remove the dots from the IP addresses
    $first_element =~ s/\.//g;
    $second_element =~ s/\.//g;

    # Sort numerically and generate the return values expected
    # of a sort routine in Perl.  For more information, see the
    # documentation of the sort() function in a Perl reference.
    if ( $first_element > $second_element ) {

	$return_value = 1;

    } elsif ( $first_element == $second_element ) {

	$return_value = 0;

    } else {

	$return_value = -1;

    }
    return $return_value;
}

sub read_conf {

    if ( -e "localscan.conf" ) {

	# If localscan.conf exists, open it.
	open(CONF, "<localscan.conf") or die;
	@conf = <CONF>;
	close(CONF);

	foreach my $line ( @conf ) {
	    chomp $line;
	    if ( $line =~ /^subnet/ ) {

		# Get the subnet data
		$line =~ s/^subnet //;
		$subnet_list = $line;

	    }
	    if ( $line =~ /^mailto/ ) {

		# Get the mailing address
		$line =~ s/^mailto //;
		$mailto = $line;

	    }
	    if ( $line =~ /^allclr/ ) {

		# Get the All Clear flag
		$line =~ s/^allclr //;
		$allclr = $line;
	    }
	    if ( $line =~ /^nmap/ ) {
	       
		# Get the nmap path
		$line =~ s/^nmap //;
		$nmap = $line;
	    }
	}

    } else {

	# So, if the subnet, mailto and/or allclr fields are not set in 
        # localscan.conf, or if localscan.conf does not exist, get them
	# from the user.
	
	get_particulars();

    }
}

sub get_particulars {

    # This gets the subnet, mailto, and/or allclr fields from the user

    unless ( $subnet ) {

	printf("What subnet \(specified by nmap scheme\) do you want to scan? ");
	$subnet_list = <STDIN>;
	chomp $subnet_list;
    }
    unless ( $mailto ) {

	printf("What's your email address? ");
	$mailto = <STDIN>;
	chomp $mailto;
    }
    unless ( $allclr ) {

	printf("Do you want to receive \"all clear\" messages (Y/N)? ");
	$allclr = <STDIN>;

	# Parse this properly
	chomp $allclr;
	if ( $allclr =~ /^n/i ) {
	    $allclr = "no";
	} else {
	    $allclr = "yes";
	}
    }
    unless ( $nmap ) {

	printf("Where is nmap located \(path only\)? ");
	$nmap = <STDIN>;
	chomp $nmap;
    }
}
